//+------------------------------------------------------------------+
//|                                                      mvt_Vlt.mq5 |
//|                        Copyright 2015, MetaQuotes Software Corp. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2015, MetaQuotes Software Corp."
#property link      "https://www.mql5.com"
#property version   "1.00"
#property description "mvt_Vlt"
#include <MovingAverages.mqh>
//---
#property indicator_separate_window
#property indicator_buffers 3
#property indicator_plots   3
//--- plot vlt_av
#property indicator_label1  "vlt_av"
#property indicator_type1   DRAW_LINE
#property indicator_color1  clrRed
#property indicator_style1  STYLE_SOLID
#property indicator_width1  1
//--- plot vlt_std
#property indicator_label2  "vlt_std"
#property indicator_type2   DRAW_LINE
#property indicator_color2  clrLawnGreen
#property indicator_style2  STYLE_SOLID
#property indicator_width2  1
//--- plot CandleHight
#property indicator_label3  "CandleHight"
#property indicator_type3   DRAW_LINE
#property indicator_color3  clrYellow
#property indicator_style3  STYLE_SOLID
#property indicator_width3  1
//--- input parameters
input int      InpTypeVlt=0;        // 0-Cndl,1-Mov
input int      InpBandsPeriod=20;
//--- global variables
int           ExtPlotBegin=0;
//--- indicator buffers
double        ExtCalcVlBuffer[];
double        ExtMovingBuffer[];
double        ExtStdDevBuffer[];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- define buffers
   SetIndexBuffer(0,ExtMovingBuffer);
   SetIndexBuffer(1,ExtStdDevBuffer);
   SetIndexBuffer(2,ExtCalcVlBuffer);
//--- set index labels
   PlotIndexSetString(0,PLOT_LABEL,"vlt_av");
   PlotIndexSetString(1,PLOT_LABEL,"vlt_std");
   PlotIndexSetString(2,PLOT_LABEL,"CandleHight");
//--- indicator name
   IndicatorSetString(INDICATOR_SHORTNAME,"mvt_Vlt");
//--- indexes draw begin settings
   ExtPlotBegin=InpBandsPeriod-1;
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,InpBandsPeriod);
   PlotIndexSetInteger(1,PLOT_DRAW_BEGIN,InpBandsPeriod);
   PlotIndexSetInteger(2,PLOT_DRAW_BEGIN,InpBandsPeriod);
////--- indexes shift settings
//   PlotIndexSetInteger(0,PLOT_SHIFT,ExtBandsShift);
//   PlotIndexSetInteger(1,PLOT_SHIFT,ExtBandsShift);
//   PlotIndexSetInteger(2,PLOT_SHIFT,ExtBandsShift);
//--- number of digits of indicator value
   IndicatorSetInteger(INDICATOR_DIGITS,_Digits+1);
//---- OnInit done   
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---
//--- variables
   int pos;
//--- check for bars count
   if(rates_total<=InpBandsPeriod)
      return(0);
//--- starting calculation
   if(prev_calculated>1) pos=prev_calculated-1;
   else pos=0;
//--- main cycle
   //for(int i=pos;i<rates_total   && !IsStopped();i++)  //     
   for(int i=pos;i<rates_total-1 && !IsStopped();i++)    //      
     {
      ExtCalcVlBuffer[i]=high[i]-low[i];
      ExtMovingBuffer[i]=SimpleMA(i,InpBandsPeriod,ExtCalcVlBuffer);
      ExtStdDevBuffer[i]=StdDev_Func(i,ExtCalcVlBuffer,ExtMovingBuffer,InpBandsPeriod);
     }
    
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
//| Calculate Standard Deviation                                     |
//+------------------------------------------------------------------+
double StdDev_Func(int position,const double &price[],const double &MAprice[],int period)
  {
//--- variables
   double StdDev_dTmp=0.0;
//--- check for position
   if(position<period) return(StdDev_dTmp);
//--- calcualte StdDev
   for(int i=0;i<period;i++) StdDev_dTmp+=MathPow(price[position-i]-MAprice[position],2);
   StdDev_dTmp=MathSqrt(StdDev_dTmp/(period-1));
//--- return calculated value
   return(StdDev_dTmp);
  }
//+------------------------------------------------------------------+
